@@ -7,6 +7,23 @@
 #define LUAJIT_VERSION_NUM 0
 #endif
 
+#ifndef PLLUA_MAX_INT_NUM
+#define PLLUA_MAX_INT_NUM LUA_MAXINTEGER
+#endif
+#ifndef PLLUA_MIN_INT_NUM
+#define PLLUA_MIN_INT_NUM LUA_MININTEGER
+#endif
+
+/*
+ * This must be able to push a value not outside PLLUA_*_INT_NUM as whatever
+ * passes for an integer in Lua.
+ */
+#if LUA_MAXINTEGER >= PLLUA_MAX_INT_NUM
+#define pllua_pushbigint(L_, v_) lua_pushinteger((L_), (lua_Integer)(v_))
+#else
+#define pllua_pushbigint(L_, v_) lua_pushnumber((L_), (lua_Number)(v_))
+#endif
+
 /*
  * We can only use 5.1 environments in a compatible way to 5.2+ uservalues if
  * we forcibly set an environment on every userdata we create.
@@ -32,9 +49,9 @@
 #else
 
 #define pllua_pushcfunction(L_,f_) do {					\
-		int rc PG_USED_FOR_ASSERTS_ONLY;				\
-		rc = lua_rawgetp((L_),LUA_REGISTRYINDEX,(f_));	\
-		Assert(rc==LUA_TFUNCTION); } while(0)
+		int rc_ PG_USED_FOR_ASSERTS_ONLY;				\
+		rc_ = lua_rawgetp((L_),LUA_REGISTRYINDEX,(f_));	\
+		Assert(rc_==LUA_TFUNCTION); } while(0)
 
 #endif
 
@@ -55,7 +72,9 @@
 
 /*
  * Handle API differences for lua_resume by emulating the 5.4 API on earlier
- * versions.
+ * versions. Also fake out the warning system on earlier versions, and provide
+ * a minimal emulation of <close> that works for normal exits (leaving error
+ * exits to be cleaned up by the GC, but that can't be helped).
  */
 #if LUA_VERSION_NUM < 504
 
@@ -80,10 +99,22 @@ pllua_resume(lua_State *L, lua_State *from, int nargs, int *nret)
 
 #define PLLUA_WARNBUF_SIZE 4
 
+#define lua_toclose(L_, i_) ((void)0)
+
+static inline void
+pllua_closevar(lua_State *L, int idx)
+{
+	if (lua_toboolean(L, idx)
+		&& luaL_callmeta(L, idx, "__close"))
+		lua_pop(L, 1);
+}
+
 #else
 
 #define PLLUA_WARNBUF_SIZE 1000
 
+#define pllua_closevar(L_, i_) ((void)0)
+
 #endif /* LUA_VERSION_NUM < 504 */
 
 #endif
